from argparse import ArgumentParser, RawDescriptionHelpFormatter
from getpass import getuser
from textwrap import dedent
from .profile import parse
from .theme import update_context
from .version import __version__


def argparser():
    ps = ArgumentParser(
        prog='segancha',
        description=dedent("""
            Generate optimized discernible color palette and themes settings

            Example:
              segancha --temperature -1 --profile demo
              segancha --background 97 --profile vscode
            """),
        formatter_class=RawDescriptionHelpFormatter)

    ps.add_argument(
        '-n',
        '--name',
        default='default',
        type=str,
        help='theme name, default: \'default\'')
    ps.add_argument(
        '-f',
        '--foreground',
        '--Lf',
        default=-1,
        dest='Lf',
        metavar='Lf',
        type=float,
        help='foreground luminosity between 0 < Lf < 100')
    ps.add_argument(
        '-b',
        '--background',
        '--Lb',
        default=-1,
        dest='Lb',
        metavar='Lb',
        type=float,
        help='background luminosity 0 < Lb < 100')
    ps.add_argument(
        '-L',
        '--palette-luminosity',
        '--L3',
        default=-1,
        dest='L3',
        metavar='L3',
        type=float,
        help='main palette luminosity 0 < L3 < 100')
    ps.add_argument(
        '--maxC',
        default=-1,
        metavar='maxC',
        type=float,
        help='maximal chroma > 0')
    ps.add_argument(
        '-T',
        '--temperature',
        default=5000,
        dest='T',
        metavar='T',
        type=float,
        help='temperature 4000 < T < 25000')
    ps.add_argument(
        '-p',
        '--profile',
        action='append',
        default=[],
        dest='profiles',
        metavar='PROFILE',
        help='profiles to be parsed')
    ps.add_argument(
        '--L2',
        default=-1,
        dest='L2',
        metavar='L2',
        type=float,
        help='(advanced) 0 < L2 < 100')
    ps.add_argument(
        '--L1',
        default=-1,
        dest='L1',
        metavar='L1',
        type=float,
        help='(advanced) 0 < L1 < 100')
    ps.add_argument(
        '--L0',
        default=-1,
        dest='L0',
        metavar='L0',
        type=float,
        help='(advanced) 0 < L0 < 100')
    ps.add_argument('-v', '--verbose', action='store_true', default=False)

    return ps


def main():
    args = argparser().parse_args()
    ctx = {
        'name': args.name,
        'version': __version__,
        'user': getuser(),
    }

    update_context(args, ctx)
    parse(args, ctx)



import os
import shutil
from pystache import Renderer
from pystache.defaults import TEMPLATE_EXTENSION

TEMPLATE_SUFFIX = os.extsep + TEMPLATE_EXTENSION
TEMPLATE_SUFFIX_LEN = len(TEMPLATE_SUFFIX)


def recursive_render(src_path,
                     dest_path,
                     context,
                     renderer=Renderer(missing_tags='strict'),
                     debug=True):
    if (os.path.isdir(src_path)):
        os.makedirs(dest_path, exist_ok=True)
        if debug:
            print(f'Enter directory {dest_path}')
        for name in os.listdir(src_path):
            child_src_path = os.path.join(src_path, name)
            child_dest_path = os.path.join(dest_path,
                                           renderer.render(name, context))
            recursive_render(child_src_path, child_dest_path, context,
                             renderer, debug)
    elif dest_path[-TEMPLATE_SUFFIX_LEN:] == TEMPLATE_SUFFIX:
        dest_path = dest_path[:-TEMPLATE_SUFFIX_LEN]
        with open(src_path, 'r') as src, open(dest_path, 'w') as dest:
            dest.write(renderer.render(src.read(), context))
        if debug:
            print(f'Parse file {dest_path}')
    else:
        shutil.copyfile(src_path, dest_path)
        if debug:
            print(f'Copy file {dest_path}')


# @param args.profiles
# @param args.verbose
def parse(args, ctx):
    srcs = ['.', os.path.join(os.path.dirname(__file__), 'data')]
    dest = f'./segancha-{args.name}'
    if not args.profiles:
        args.profiles = ['demo']
        print('Available builtin profiles:')
        for d in srcs[1:]:
            if not os.path.isdir(d):
                continue
            for n in os.listdir(d):
                print(f'  {n}')
    for profile in args.profiles:
        profile_srcs = [os.path.join(d, profile) for d in srcs]
        found = False
        for profile_src in profile_srcs:
            if os.path.exists(profile_src):
                found = True
                break
        if not found:
            raise OSError(f'Cannot find profile \'{profile}\'')
        profile_dest = os.path.join(dest, profile)
        print(f'Parse profile \'{profile}\' into {profile_dest}')
        recursive_render(profile_src, profile_dest, ctx, debug=args.verbose)



from itertools import product
from .native import LABtoRGB, LCHtoLAB, maxChroma, segancha, IlluminantD

# D50_x = 0.3457048
SEMANTIC_HUE = {'red': 30, 'yellow': 80, 'green': 120, 'blue': 260}


def hex(x):
    return '{:02x}'.format(min(round(256 * max(0, x)), 255)).upper()


def rgb_hex(rgb):
    return ''.join([hex(x) for x in rgb])


def interpolate2D(x1, y1, z1, x2, y2, z2, x3, y3, z3):
    d = x2 * y1 - x3 * y1 - x1 * y2 + x3 * y2 + x1 * y3 - x2 * y3
    c = x3 * y2 * z1 - x2 * y3 * z1 - x3 * y1 * z2 + x1 * y3 * z2 + x2 * y1 * z3 - x1 * y2 * z3
    b = x3 * z1 + x1 * z2 - x3 * z2 - x1 * z3 + x2 * z3 - x2 * z1
    a = y2 * z1 - y3 * z1 - y1 * z2 + y3 * z2 + y1 * z3 - y2 * z3
    return lambda x, y: (c - y * b - x * a) / d


# @param args.Lf
# @param args.Lb
# @param args.L
# @param args.maxC
# @param args.T
# @param args.verbose
def update_context(args, ctx):
    if args.Lb < 0:
        if args.Lf < 0:
            args.Lf = 93
        args.Lb = 100 - args.Lf
    elif args.Lf < 0:
        args.Lf = 100 - args.Lb

    ctx['fg-hex'] = rgb_hex(LABtoRGB((IlluminantD(args.T, args.Lf))))
    ctx['ui-theme'] = 'vs-dark' if args.Lb < 50 else 'vs'

    # L3: text color, very close to Lf
    if args.L3 < 0:
        args.L3 = interpolate2D(0, 100, 45, 100, 0, 80, 60, 60, 60)(args.Lf,
                                                                    args.Lb)
    # L2: UI line/background, large inter-distance, close to Lf
    if args.L2 < 0:
        args.L2 = interpolate2D(0, 100, 50, 100, 0, 70, 60, 60, 60)(args.Lf,
                                                                    args.Lb)
    # L1: indicator color, large inter-distance, close to Lb
    if args.L1 < 0:
        args.L1 = interpolate2D(0, 100, 70, 100, 0, 55, 60, 60, 60)(args.Lf,
                                                                    args.Lb)
    # L0: background color, very close to Lb
    if args.L0 < 0:
        args.L0 = interpolate2D(0, 100, 90, 100, 0, 15, 60, 60, 60)(args.Lf,
                                                                    args.Lb)

    if (args.verbose):
        print(
            f'Init parameters: Lf={args.Lf} L3={args.L3} args.L2={args.L2} args.L1={args.L1} '
            + f'args.L0={args.L0} Lb={args.Lb} T={args.T}')

    sgn = 1 if args.Lf > args.Lb else -1
    palette3 = segancha(
        7,
        L=args.L3,
        maxC=args.maxC,
        fixed=[IlluminantD(args.T, args.L3)],
        quiet=not args.verbose)

    for i, rgb in enumerate(palette3['rgb']):
        ctx[f'main-{i}-hex'] = rgb_hex(rgb)

    for (name, hue), (level,
                      L) in product(SEMANTIC_HUE.items(),
                                    enumerate([args.L0, args.L1, args.L2])):
        ctx[f'{name}-{level}-hex'] = rgb_hex(
            LABtoRGB(LCHtoLAB(maxChroma([L, 0, hue], maxC=args.maxC))))

    for i, delta in enumerate([15, 25, 60]):
        ctx[f'line-{i}-hex'] = rgb_hex(
            LABtoRGB(IlluminantD(args.T, args.Lb + sgn * delta)))

    for i, delta in enumerate([0, 5, 12]):
        ctx[f'bg-{i}-hex'] = rgb_hex(
            LABtoRGB(IlluminantD(args.T, args.Lb + sgn * delta)))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from distutils.version import LooseVersion
from functools import reduce
from glob import glob
from multiprocessing import cpu_count
from platform import system
from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext
import os
import re
import subprocess
import sys


class CMakeExtension(Extension):
    def __init__(self, name, sourcedir='', **kwargs):
        Extension.__init__(self, name, **kwargs)
        self.sourcedir = os.path.abspath(sourcedir)


class CMakeBuild(build_ext):
    def run(self):
        try:
            out = subprocess.check_output(['cmake', '--version'])
        except OSError:
            raise RuntimeError(
                "CMake must be installed to build the following extensions: " +
                ", ".join(e.name for e in self.extensions))

        if system() == "Windows":
            cmake_version = LooseVersion(
                re.search(r'version\s*([\d.]+)', out.decode()).group(1))
            if cmake_version < '3.1.0':
                raise RuntimeError("CMake >= 3.1.0 is required on Windows")

        for ext in self.extensions:
            self.build_extension(ext)

    def build_extension(self, ext):
        extdir = os.path.abspath(
            os.path.dirname(self.get_ext_fullpath(ext.name)))
        cmake_args = [
            '-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=' + extdir,
            '-DPYTHON_EXECUTABLE=' + sys.executable
        ]

        cfg = 'Debug' if self.debug else 'Release'
        build_args = ['--config', cfg]

        if system() == "Windows":
            cmake_args += [
                '-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{}={}'.format(
                    cfg.upper(), extdir)
            ]
            if sys.maxsize > 2**32:
                cmake_args += ['-A', 'x64']
            build_args += ['--', '/m']
        else:
            cmake_args += ['-DCMAKE_BUILD_TYPE=' + cfg]
            build_args += ['--', '-j{0}'.format(cpu_count())]

        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)
        env = os.environ
        subprocess.check_call(
            ['cmake', ext.sourcedir] + cmake_args,
            cwd=self.build_temp,
            env=env)
        subprocess.check_call(
            ['cmake', '--build', '.'] + build_args, cwd=self.build_temp)


with open('python/version.py') as f:
    exec(f.read())

setup(
    name='segancha',
    version=__version__,  # noqa: F821
    description='Segancha colors model',
    install_requires=['pystache'],
    packages=['segancha'],
    package_dir={'segancha': 'python'},
    ext_modules=[
        CMakeExtension(
            'segancha.native',
            sources=[
                f
                for f in
                reduce(lambda l, d: l + glob(f'{d}/**', recursive=True),
                       ['cmake', 'include', 'src'], ['CMakeLists.txt'])
                if os.path.isfile(f)
            ])
    ],
    package_data={
        'segancha': [
            os.path.relpath(p, 'python')
            for p in glob('python/data/**', recursive=True)
        ]
    },
    entry_points={'console_scripts': [
        'segancha = segancha.cli:main',
    ]},
    cmdclass=dict(build_ext=CMakeBuild),
    url='https://github.com/gywn/segancha',
    download_url='https://github.com/gywn/segancha/archive/v0.0.1.tar.gz')
